module break_model_2

using Distributions, StatsBase, Combinatorics, Printf, Distributed, SpecialFunctions, Random, LinearAlgebra

mutable struct break_model
    G0::Int64 # Number of burn-ins
    G::Int64 # Number of posterior draws
    k_pot::Int64 # Number of potential variables
    T::Int64 # Total number off time periods
    xj_sim_g::Array{Float64,1} # Indicator vector for variable selection
    qj_sim_g::Array{Float64,2} # Indicator matrix for break date selection
    x_pot::Array{Float64,2} # Matrix containing all potential variables
    Y::Array{Float64,1} # Dependent variable
    tau_prior::Array{Float64,1} # Prior used in break date selection
    model_size_prior::Array{Float64,2} # Prior used in break date selection
   
    d_vec::Array{Float64,1} # cI prior
    int_prior_var::Float64  # Prior on intercept variance
    m::Float64  # Prior on first error precision
    v::Float64 # Prior on first error precision
    mb::Float64 # Prior on 2+ error precision
    vb::Float64 # Prior on 2+ error precision
    minp::Int64 # Minimum number of periods before next break
    max_m::Int64 # Maximum bumber of breaks
    total_draws::Int64 # Total draws (G0+G)
    nbhood::Int64 # Neighborhood size
    seed::Int64 # Random seed used
    thin::Int64
end

mutable struct break_model_output_post
	x_sampled::Array{Float64,1}
	qj_sim_temp::Array{Float64,2} 
	num_break_sampled::Array{Float64,1} # Number of break dates in each draw
	num_break_param_sampled::Array{Float64,2} # Number of break dates in each draw, by variable
	break_date_sampled::Array{Float64,2} # Indicators for break dates in each draw

	x_sto::Array{Float64,2} # Matrix containing variable indicator in each posterior draw
	qj_sto::Array{Float64,3} # Tensor containing break date indicators in each posterior draw
	B_sto::Array{Float64,3} # Tensor containing regression parameter estimates in each posterior draw
	H_sto::Array{Float64,2} # Matrix containing error precision estimates in each posterior draw
    ln_lik_sto::Array{Float64,1}

	x_pre::Array{Float64,2} # Matrix containing variable indicator in each burn-in draw
	qj_pre::Array{Float64,3} # Tensor containing break date indicators in each burn-in draw
	B_pre::Array{Float64,3} # Tensor containing regression parameter estimates in each burn-in draw
	H_pre::Array{Float64,2} # Matrix containing error precision estimates in each burn-in draw
	seed::Int64 # Seed number used
    x_accept::Int64
    A_accept::Int64
    ln_lik_all::Array{Float64,1} 
    r_all::Array{Float64,1} 
end


function gibbs_break(model::break_model)
    # Define parameters. These are retrieved from the break_model

    total_draws = model.total_draws
    nbhood = model.nbhood
    qj_sim_g = model.qj_sim_g
    xj_sim_g = model.xj_sim_g
    k_pot = model.k_pot
    T = model.T
    Y = model.Y
    x_pot = model.x_pot
    tau_prior = model.tau_prior
    model_size_prior = model.model_size_prior
    int_prior_var = model.int_prior_var
    m = model.m
    v = model.v
    mb = model.mb
    vb = model.vb
    minp = model.minp
    max_m=model.max_m
    G = model.G
    G0 = model.G0
    seed=model.seed

    d_vec = model.d_vec

    # Set random seed
    Random.seed!(seed)

    thin = model.thin;
    g = Int64(G./thin)
    g0 = Int64(G0./thin)
    
    # Initialize storage matrices
    qj_sim_temp = qj_sim_g[findall([1;xj_sim_g;1] .!=0),:] # Initial break-date indiactor matrix
    x_sampled = zeros(k_pot)
    num_break_sampled = zeros(g)
    num_break_param_sampled = zeros(k_pot+2,g);
    break_date_sampled = zeros(T);

    x_sto = zeros(k_pot,g);
	qj_sto = zeros(k_pot+2,T,g);
    B_sto = zeros(k_pot+1,T,g);
	H_sto = zeros(T,g);
    d_sto = zeros(3,g)
    σd_sto = zeros(g)
    mb_sto = zeros(g)
    σb_sto = zeros(g)
    ln_lik_sto = zeros(g)
    ln_lik_all = zeros(G+G0)
    r_all = zeros(G+G0)

	x_pre = zeros(k_pot,g0);
	qj_pre = zeros(k_pot+2,T,g0);
    B_pre = zeros(k_pot+1,T,g0);
	H_pre = zeros(T,g0);
    d_pre = zeros(3,g0)
    σd_pre = zeros(g0)
    mb_pre = zeros(g0)
    σb_pre = zeros(g0)
    
    num_breaks_record = 0.0
    num_var_breaks_record = 0.0
    num_reg_record = 0 
    avg_breaks=0
    avg_reg=0
    avg_var_breaks=0
    # Begin sampler
    x_accept=0
    A_accept=0
    for sim = 1:total_draws
        # Print progress
	    if mod(sim,50000)==0
	    	@printf "Iteration = %d" sim 
	    	@printf " of %d\n" total_draws
            @printf " %f avg. total breaks\n" avg_breaks
            @printf " %f avg. AR terms\n" avg_reg
            #@printf " %f avg. var breaks\n" avg_var_breaks
	    end

        # Draw included variables
        (xj_sim_g1, xa) = break_model_2.gen_xj_new(Y, xj_sim_g, qj_sim_g, x_pot, tau_prior, model_size_prior, d_vec, int_prior_var, m, v, mb, vb)
        x_accept += xa

        # Draw break dates & coefficient estimates for each variable
        r = rand()
        if r<0.5
        	(qj_sim_g1, B_g1, H_g1, B_raw,h_raw,ln_fy_proc,ln_tau,qa,r) = break_model_2.gen_qj_single_move(Y, xj_sim_g1, qj_sim_g, x_pot, minp, nbhood, tau_prior, max_m, model_size_prior, d_vec, int_prior_var, m, v, mb, vb)
        else
        	(qj_sim_g1, B_g1, H_g1, B_raw,h_raw,ln_fy_proc,ln_tau,qa,r) = break_model_2.gen_qj_multiple_move(Y, xj_sim_g1, qj_sim_g, x_pot, minp, nbhood, tau_prior, max_m, model_size_prior, d_vec, int_prior_var, m, v, mb, vb)
        end
        A_accept += qa

        num_breaks_simg1 = length(findall(sum(qj_sim_g1,dims=1).>0))
        num_breaks_record +=  num_breaks_simg1
        avg_breaks = num_breaks_record./sim
        #@printf " %f breaks drawn \n" num_breaks_simg1

        num_var_breaks_simg1 = length(findall(qj_sim_g1[end,:][:] .>0))
        num_var_breaks_record +=  num_var_breaks_simg1
        avg_var_breaks = num_var_breaks_record./sim
        #@printf " %f var breaks drawn \n" num_var_breaks_simg1

        num_reg_simg1 = sum(xj_sim_g1)
        num_reg_record += num_reg_simg1
        avg_reg = num_reg_record./sim
        #@printf " %f AR terms included \n" num_reg_simg1

        #@printf " marginal likelihood = %f \n" ln_fy_proc

        # Store results of draw
        if (sim > G0) && (mod(sim,thin)==0)
        	place = Int64((sim-G0)/thin)

            x_sampled = x_sampled + xj_sim_g1
            qj_sim_temp = qj_sim_g1[findall([1;xj_sim_g1;1] .!=0),:]
            num_break_sampled[place] = length(findall(sum(qj_sim_temp,dims=1)[:] .!=0))
            num_break_param_sampled[:,place] = (sum(qj_sim_g1,dims=2).*[1;xj_sim_g1;1]);
            break_date_sampled = break_date_sampled + (sum(qj_sim_temp,dims=1)'.>0);

            x_sto[:,place] = xj_sim_g1;
        	qj_sto[:,:,place] = qj_sim_g1;
            B_sto[:,:,place] = B_g1;
        	H_sto[:,place] = H_g1;
            ln_lik_sto[place] = ln_fy_proc;
        elseif (mod(sim,thin)==0)
        	place = Int64(sim./thin)

        	x_pre[:,place] = xj_sim_g1;
        	qj_pre[:,:,place] = qj_sim_g1;
            B_pre[:,:,place] = B_g1;
        	H_pre[:,place] = H_g1;
        end
        ln_lik_all[sim] = ln_fy_proc
        r_all[sim] = r

        # Set "old" variable indiator and break dates for use in next draw
        xj_sim_g = deepcopy(xj_sim_g1)
        qj_sim_g = deepcopy(qj_sim_g1)
    end
    
    # Save results using the "break_model_output_post" type
    post = break_model_output_post(x_sampled, qj_sim_temp, num_break_sampled, num_break_param_sampled, break_date_sampled,x_sto,qj_sto,B_sto,H_sto,ln_lik_sto,x_pre,qj_pre,B_pre,H_pre,seed,x_accept,A_accept,
    ln_lik_all,r_all)

    # Return results
    return post
end

function gen_xj_new(Y::Array{Float64,1},xj_in::Array{Float64,1},qj_in::Array{Float64,2},
        x_pot::Array{Float64,2},tau_prior::Array{Float64,1},model_size_prior::Array{Float64,2},
        d_vec::Array{Float64,1},int_prior_var::Float64,m::Float64,v::Float64,mb::Float64,vb::Float64)
    ###############################################################
    # Set some parameter values
    ###############################################################
    # Total number of time periods
    T = length(Y)
    # Total number of possible variables (excluding intercept)
    k_pot = size(x_pot,2)
    # Indicator vector of currently included variables
    xj_star = deepcopy(xj_in)

    # Draw an AR order at random from 0,1,...k_pot
    u = ceil(rand()*(k_pot+1))-1
    u = convert(Int64,u)

    # Compare to current AR model. If identical, return current model and end function
    if u.==sum(xj_in)
        return (xj_in, 1)
    end

    # Can't remove a variable that experiences a break. If a variable that breaks is
    # excluded from the proposed model, return the current model and end function
    qj_sim_AR = qj_in[findall([0;xj_in] .!=0 ),:]
    num_break_AR = sum(qj_sim_AR,dims=2)[:]
    AR_breaks = findall(num_break_AR .> 0)

    if length(AR_breaks)==0
        # no minimum, so continue 
    else
        min_AR_length = AR_breaks[end]
        if u .<  min_AR_length
            return (xj_in, 0)
        end
    end


    ###############################################################
    # Compute Marginal Likelihood of "current" model
    ###############################################################

    # Total number of "current" included variables
    k_in = convert(Int64,sum(xj_in))
    # Break date indicators only for currently included variables
    qj_sim_reduced = qj_in[findall([1;xj_in] .!=0),:]
    # Total number of breaks in each variable
    num_break_param = sum(qj_sim_reduced,dims=2)

    # Create "current" X matrix
    x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_sim_reduced,k_in,num_break_param,T)

    # Indicator vector of break dates
    var_break_indic = qj_in[k_pot+2,:][:];

    # If no variance breaks, use analytical calculation.
    # Else, use the Chib simulation method to estimate.
    if sum(var_break_indic)==0
        (ln_fy_sim, B_in, H_in) = ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
    else
        (ln_fy_sim, B_in, H_in) = ln_marg_lik_gs(Y, x_proc, var_break_indic, 
                                d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end

    # Model Prior
    num_break_sim = length(findall(sum(qj_sim_reduced,dims=1)[:] .!=0))
    num_param_sim = size(x_proc,2) + sum(var_break_indic) + 1;
    
    model_size_place = findall(model_size_prior[:,1].==num_param_sim)[1]
    model_size_sim = model_size_prior[model_size_place,2]

    # Marginal likelihood of "current" model
    ln_model_prob_sim = ln_fy_sim + log.(tau_prior[num_break_sim+1]) + log(model_size_sim)

    ###############################################################
    # "Proposal" model parameters
    ###############################################################
    xj_star = zeros(k_pot)
    xj_star[1:u] .= 1
    # Total number of included variables in "proposal" model
    k_star = convert(Int64,sum(xj_star))
    # Break date indicators only for "proposed" included variables
    qj_star_reduced = qj_in[findall([1;xj_star] .!=0),:]
    # Total number of breaks in each variable under "proposal" model
    num_break_param = sum(qj_star_reduced,dims=2)

    # Create "proposal" X matrix
    x_proc = make_X_break(x_pot,xj_star,qj_star_reduced,k_star,num_break_param,T)

    ###############################################################
    # Compute Marginal Likelihood of "proposal" model
    ###############################################################
    # Compute number of variance breaks
    var_break_indic = qj_in[k_pot+2,:][:]

    # If no variance breaks, use analytical calculation.
    # Else, use the Chib simulation method to estimate.
    if sum(var_break_indic)==0
        (ln_fy_star, B_star, H_star) = ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_star, num_break_param);
    else
        (ln_fy_star, B_star, H_star) = ln_marg_lik_gs(Y, x_proc, var_break_indic, 
                                    d_vec, int_prior_var, m, v, mb, vb, k_star, num_break_param);
    end

    # Number of breaks in "proposal" model
    num_break_star = length(findall(sum(qj_star_reduced,dims=1)[:] .!=0))
    num_param_star = size(x_proc,2) + sum(var_break_indic) + 1;

    model_size_place = findall(model_size_prior[:,1].==num_param_star)[1]
    model_size_star = model_size_prior[model_size_place,2]

    # Marginal likelihood of "proposal" model
    ln_model_prob_star = ln_fy_star + log.(tau_prior[num_break_star+1]) + log(model_size_star)

    ###############################################################
    # Compute Model Probabilities and Draw Model
    ############################################################### 
    # Acceptance probability based on marginal likelihoods
    ln_diff = ln_model_prob_star-ln_model_prob_sim
    prob_acc = minimum([exp(ln_diff);1])
    # Draw random number
    u = rand()
    # Use random number to select a model.
    MH_a = 0
    if u<=prob_acc;
        xj_update = xj_star;
        MH_a = 1;
    else
        xj_update = xj_in;
    end

    # Return chosen indicator vector
    return (xj_update, MH_a)
end


function gen_xj(Y::Array{Float64,1},xj_in::Array{Float64,1},qj_in::Array{Float64,2},
        x_pot::Array{Float64,2},tau_prior::Array{Float64,1},model_size_prior::Array{Float64,2},
        d_vec::Array{Float64,1},int_prior_var::Float64,m::Float64,v::Float64,mb::Float64,vb::Float64)
    ###############################################################
    # Set some parameter values
    ###############################################################
    # Total number of time periods
    T = length(Y)
    # Total number of possible variables (excluding intercept)
    k_pot = size(x_pot,2)
    # Indicator vector of currently included variables
    xj_star = deepcopy(xj_in)

    # Draw a variable from the total list at random
    u = ceil(rand()*k_pot)
    u = convert(Int64,u)

    # Count number of models in a neighborhood (defined by proposal density) of current model
    break_ind_vec = (sum(qj_in,dims=2) .!= 0);
    break_ind_vec = break_ind_vec[2:k_pot+1];
    poss_mod_mat = [xj_in+break_ind_vec collect(1:1:k_pot)];
    poss_mod_mat = poss_mod_mat[poss_mod_mat[:,1].<2,:];
    num_mod_g = size(poss_mod_mat,1);


    # Generate proposed X from proposal density
    u = convert(Int64,ceil(rand()*(num_mod_g+1)));
    if u<=num_mod_g
        place = convert(Int64,poss_mod_mat[u,2])
        if xj_in[place]==1
            xj_star[u]=0;
        else
            xj_star[u]=1;
        end
    end


    # Total number of "current" included variables
    k_in = convert(Int64,sum(xj_in))
    # Break date indicators only for currently included variables
    qj_sim_reduced = qj_in[findall([1;xj_in] .!=0),:]
    # Total number of breaks in each variable
    num_break_param = sum(qj_sim_reduced,dims=2)

    # Create "current" X matrix
    x_proc = make_X_break(x_pot,xj_in,qj_sim_reduced,k_in,num_break_param,T)

    ###############################################################
    # Compute Marginal Likelihood of "current" model
    ###############################################################
    # Indicator vector of break dates
    var_break_indic = qj_in[k_pot+2,:][:];

    # If no variance breaks, use analytical calculation.
    # Else, use the Chib simulation method to estimate.
    if sum(var_break_indic)==0
        (ln_fy_sim, B_in, H_in) = ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
    else
        (ln_fy_sim, B_in, H_in) = ln_marg_lik_gs(Y, x_proc, var_break_indic, 
                                d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end

    # Model Prior
    num_break_sim = length(findall(sum(qj_sim_reduced,dims=1)[:] .!=0))
    num_param_sim = size(x_proc,2) + sum(var_break_indic) + 1;
    
    model_size_place = findall(model_size_prior[:,1].==num_param_sim)[1]
    model_size_sim = model_size_prior[model_size_place.-num_break_sim,2]

    # Marginal likelihood of "current" model
    ln_model_prob_sim = ln_fy_sim + log.(tau_prior[num_break_sim+1]) + log(model_size_sim)

    ###############################################################
    # "Proposal" model parameters
    ###############################################################
    # Total number of included variables in "proposal" model
    k_star = convert(Int64,sum(xj_star))
    # Break date indicators only for "proposed" included variables
    qj_star_reduced = qj_in[findall([1;xj_star] .!=0),:]
    # Total number of breaks in each variable under "proposal" model
    num_break_param = sum(qj_star_reduced,dims=2)

    # Create "proposal" X matrix
    x_proc = make_X_break(x_pot,xj_star,qj_star_reduced,k_star,num_break_param,T)

    ###############################################################
    # Compute Marginal Likelihood of "proposal" model
    ###############################################################
    # Compute number of variance breaks
    var_break_indic = qj_in[k_pot+2,:][:]

    # If no variance breaks, use analytical calculation.
    # Else, use the Chib simulation method to estimate.
    if sum(var_break_indic)==0
        (ln_fy_star, B_star, H_star) = ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_star, num_break_param);
    else
        (ln_fy_star, B_star, H_star) = ln_marg_lik_gs(Y, x_proc, var_break_indic, 
                                    d_vec, int_prior_var, m, v, mb, vb, k_star, num_break_param);
    end

    # Number of breaks in "proposal" model
    num_break_star = length(findall(sum(qj_star_reduced,dims=1)[:] .!=0))
    num_param_star = size(x_proc,2) + sum(var_break_indic) + 1;

    model_size_place = findall(model_size_prior[:,1].==num_param_star)[1]
    model_size_star = model_size_prior[model_size_place.-num_break_star,2]

    # Marginal likelihood of "proposal" model
    ln_model_prob_star = ln_fy_star + log.(tau_prior[num_break_star+1]) + log(model_size_star)

    ###############################################################
    # Compute Model Probabilities and Draw Model
    ############################################################### 
    # Acceptance probability based on marginal likelihoods
    ln_diff = ln_model_prob_star-ln_model_prob_sim
    prob_acc = minimum([exp(ln_diff);1])
    # Draw random number
    u = rand()
    # Use random number to select a model.
    if u<=prob_acc;
        xj_update = xj_star;
    else
        xj_update = xj_in;
    end

    # Return chosen indicator vector
    return xj_update
end


function gen_qj_multiple_move(Y::Array{Float64,1}, xj_in::Array{Float64,1}, qj_in::Array{Float64,2}, 
x_pot::Array{Float64,2}, minp::Int64, nbhood::Int64, tau_prior::Array{Float64,1}, 
max_m::Int64, model_size_prior::Array{Float64,2}, d_vec::Array{Float64,1}, int_prior_var::Float64, 
m::Float64, v::Float64, mb::Float64, vb::Float64)
    ###############################################################
    # Set some parameter values
    ###############################################################
    # Define constants
    k_in = length(findall(xj_in.>0))
    qj_reduced = qj_in[findall([1;xj_in;1] .!=0),:]
    qj = deepcopy(qj_reduced)
    (kk,T) = size(qj)
    r=0;

    ###############################################################
    # Find marginal likelihood of current model
    ###############################################################
    num_break_param = sum(qj[1:k_in+1,:],dims=2)
    x_proc = break_model_2.make_X_break(x_pot,xj_in,qj,k_in,num_break_param,T)
    var_break_indic = qj[k_in+2,:][:]
    if sum(var_break_indic)==0
        (ln_fy_proc,dont_use,dont_use) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        else
        (ln_fy_proc,dont_use,dont_use) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end
    num_param = size(x_proc,2) + sum(var_break_indic) + 1;
    num_break = Int64(length(findall(sum(qj,dims=1)[:] .!=0)))
    model_size_place = findall(model_size_prior[:,1].==num_param)[1]
    model_size_sim = model_size_prior[model_size_place,2]
    lnML_current = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)

    ###############################################################
    # Generate Proposal Matrix
    ###############################################################
    breakdates = findall(sum(qj,dims=1)[:] .>0)
	num_break = Int64(length(findall(sum(qj,dims=1)[:] .!=0)))
	Aprop = zeros(kk,T)
	if num_break>0
        # Move "nr" breakdates
        nr = Int64(ceil(rand().*num_break))
        breakdates_samp = sample(breakdates,nr,replace=false)
		for i=1:nr
			breakdate = breakdates_samp[i]
			break_type = qj[:,breakdate]
			places = collect(breakdate-nbhood:1:breakdate+nbhood)
			keep = findall(places.!=breakdate)
			places = places[keep]
			place_break = sample(places)[1]
			Aprop[:,place_break] = break_type
		end

        # Keep all other breakdates in their original place
        for i=1:num_break
            if sum(breakdates[i].==breakdates_samp)==0
                breakdate = breakdates[i]
                break_type = qj[:,breakdate]
                Aprop[:,breakdate] = break_type
            end
        end

	end

    break_location = findall(sum(Aprop,dims=1)[:] .!=0)
    diff_break_location = diff(break_location)

    ###############################################################
    # Compute ML of Proposal and Draw
    ###############################################################
    qa = 0
    if (sum(break_location.<=minp)>0) || (sum(break_location.>T-minp)>0) || (sum(diff_break_location.<=minp)>0) || (Aprop==qj)
        # If proposal is ineligible OR identical to the current model, keep the current model
    else
        qj_prop = deepcopy(Aprop)
        
        num_break_param = sum(qj_prop[1:k_in+1,:],dims=2)

        x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_prop,k_in,num_break_param,T)
        var_break_indic = qj_prop[k_in+2,:][:]
        if sum(var_break_indic)==0
            (ln_fy_proc,dont_use,dont_use,flag) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        else
            (ln_fy_proc,dont_use,dont_use,flag) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
        end

        if flag==1
            lnML_prop = -1e20
        else
            num_param = size(x_proc,2) + sum(var_break_indic) + 1;
            num_break = Int64(length(findall(sum(qj_prop,dims=1)[:] .!=0)))

            model_size_place = findall(model_size_prior[:,1].==num_param)[1]
            model_size_sim = model_size_prior[model_size_place,2]
            # Marginal likelihood of "proposal" model

            if num_break>max_m
                lnML_prop = -1e20
            else
                lnML_prop = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)
            end
        end
            
        # Compute acceptance probability
        rel_ML = exp(lnML_prop.-lnML_current)
        accept_prob = minimum([1.0;rel_ML])
        r = rand()
        if r<=accept_prob
            qj = deepcopy(qj_prop)
            lnML_current = deepcopy(lnML_prop)
            qa = 1
        end
    end 

    ###############################################################
    # Save new break date indicator matrix
    ###############################################################
    # Total number of possible variables
    k_pot = size(x_pot,2);
    # Initialize empty break date indicator matrix
    qj_new = zeros(k_pot+2,T);
    # Fill in the relevant rows of this matrix with new break dates
    qj_new[findall([1;xj_in;1] .!=0),:] = qj

    ###############################################################
    # Compute regression estimates
    ###############################################################
    # Total number of break dates
    num_break_param = sum(qj[1:k_in+1,:],dims=2)
    # All included columns of "X"

    # Create selected X matrix
    x_proc = make_X_break(x_pot,xj_in,qj,k_in,num_break_param,T)

    # Compute Marginal Likelihood of selected model
    var_break_indic = qj[k_in+2,:][:]
    # If no variance breaks, use analytical calculation.
    # Else, use the Chib simulation method to estimate.
    if sum(var_break_indic)==0
        (ln_fy_proc,B_in,H_in) = ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        H_med = H_in[1]
    else
        (ln_fy_proc,B_in,H_in,H_med) = ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end
    num_param = size(x_proc,2) + sum(var_break_indic) + 1;
    num_break = Int64(length(findall(sum(qj,dims=1)[:] .!=0)))

    model_size_place = findall(model_size_prior[:,1].==num_param)[1]
    model_size_sim = model_size_prior[model_size_place,2]
    # Marginal likelihood of new model
    lnML_prop = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)


    # Select rows of included variables in break date indicator matrix
    qj_sim_reduced = qj_new[findall([1;xj_in] .!=0),:]

    B_update = gen_B(xj_in,k_in,qj_sim_reduced,B_in,T,k_pot);
    # Compute estimate of error precission
    H_update = H_in;

    ###############################################################
    # Return (1) break date indicator matrix, (2) regression
    # parameter estimaes, (3) Error precision estimate
    ###############################################################
    return qj_new, B_update, H_update, B_in, H_med, lnML_prop, log.(tau_prior[num_break+1]), qa, r

end


function gen_qj_single_move(Y::Array{Float64,1}, xj_in::Array{Float64,1}, qj_in::Array{Float64,2}, 
x_pot::Array{Float64,2}, minp::Int64, nbhood::Int64, tau_prior::Array{Float64,1}, 
max_m::Int64, model_size_prior::Array{Float64,2}, d_vec::Array{Float64,1}, int_prior_var::Float64, 
m::Float64, v::Float64, mb::Float64, vb::Float64)
    ###############################################################
    # Set some parameter values
    ###############################################################
    # Define constants
    k_in = length(findall(xj_in.>0))
    qj_reduced = qj_in[findall([1;xj_in;1] .!=0),:]
    qj = deepcopy(qj_reduced)
    (kk,T) = size(qj)
    r=0

    ###############################################################
    # Find marginal likelihood of current model
    ###############################################################
    num_break_param = sum(qj[1:k_in+1,:],dims=2)
    x_proc = break_model_2.make_X_break(x_pot,xj_in,qj,k_in,num_break_param,T)
    var_break_indic = qj[k_in+2,:][:]
    if sum(var_break_indic)==0
        (ln_fy_proc,dont_use,dont_use) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        else
        (ln_fy_proc,dont_use,dont_use) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end
    num_param = size(x_proc,2) + sum(var_break_indic) + 1;
    num_break = Int64(length(findall(sum(qj,dims=1)[:] .!=0)))
    model_size_place = findall(model_size_prior[:,1].==num_param)[1]
    model_size_sim = model_size_prior[model_size_place,2]
    lnML_current = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)

    ###############################################################
    # Generate Proposal Matrix
    ###############################################################
    permmat = permn([0;1],kk)';
    permmat = permmat[:,2:size(permmat,2)];

    indx = Int64(ceil(rand()*(T-(minp-1))));
    numperm = ((2.0.^kk)-1)*minp + 1;

    choose = ceil(rand()*numperm)
    if choose==1
        Atilde = zeros(kk,minp);
    else
        choose1 = Int64(ceil(rand()*minp));
        choose2 = Int64(ceil(rand()*((2.0.^kk)-1)));

        Atilde = zeros(kk,minp);
        Atilde[:,choose1] = permmat[:,choose2];
    end

    Aprop = deepcopy(qj);
    Aprop[:,indx:indx+(minp-1)] = Atilde

    break_location = findall(sum(Aprop,dims=1)[:] .!=0)
    diff_break_location = diff(break_location)

    ###############################################################
    # Compute ML of Proposal and Draw
    ###############################################################
    qa = 0
    if (sum(break_location.<=minp)>0) || (sum(break_location.>T-minp)>0) || (sum(diff_break_location.<=minp)>0) || (Aprop==qj)
        # If proposal is ineligible OR identical to the current model, keep the current model
    else
        qj_prop = deepcopy(Aprop)
        
        num_break_param = sum(qj_prop[1:k_in+1,:],dims=2)

        x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_prop,k_in,num_break_param,T)
        var_break_indic = qj_prop[k_in+2,:][:]
        if sum(var_break_indic)==0
            (ln_fy_proc,dont_use,dont_use,flag) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        else
            (ln_fy_proc,dont_use,dont_use,flag) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
        end

        if flag==1
            lnML_prop = -1e20
        else
            num_param = size(x_proc,2) + sum(var_break_indic) + 1;
            num_break = Int64(length(findall(sum(qj_prop,dims=1)[:] .!=0)))
            num_break_old = Int64(length(findall(sum(qj,dims=1)[:] .!=0)))

            model_size_place = findall(model_size_prior[:,1].==num_param)[1]
            model_size_sim = model_size_prior[model_size_place,2]
            # Marginal likelihood of "proposal" model
            if num_break>max_m
                lnML_prop = -1e20
            else
                lnML_prop = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)
                if num_break > num_break_old
                    prob_remove = (minp/(T-2*minp))*(1/numperm)
                    prob_add = ((T-2*minp-minp)/(T-2*minp))*((numperm-1)/numperm)
                    lnML_prop = deepcopy(lnML_prop) #- log(prob_add/prob_remove)
                end
            end
        end
            
        # Compute acceptance probability
        rel_ML = exp(lnML_prop.-lnML_current)
        accept_prob = minimum([1.0;rel_ML])
        r = rand()
        if r<=accept_prob
            qj = deepcopy(qj_prop)
            lnML_current = deepcopy(lnML_prop)
            qa = 1
        end
    end 

    ###############################################################
    # Save new break date indicator matrix
    ###############################################################
    # Total number of possible variables
    k_pot = size(x_pot,2);
    # Initialize empty break date indicator matrix
    qj_new = zeros(k_pot+2,T);
    # Fill in the relevant rows of this matrix with new break dates
    qj_new[findall([1;xj_in;1] .!=0),:] = qj

    ###############################################################
    # Compute regression estimates
    ###############################################################
    # Total number of break dates
    num_break_param = sum(qj[1:k_in+1,:],dims=2)
    # All included columns of "X"

    # Create selected X matrix
    x_proc = make_X_break(x_pot,xj_in,qj,k_in,num_break_param,T)

    # Compute Marginal Likelihood of selected model
    var_break_indic = qj[k_in+2,:][:]
    # If no variance breaks, use analytical calculation.
    # Else, use the Chib simulation method to estimate.
    if sum(var_break_indic)==0
        (ln_fy_proc,B_in,H_in) = ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        H_med = H_in[1]
    else
        (ln_fy_proc,B_in,H_in,H_med) = ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end
    num_param = size(x_proc,2) + sum(var_break_indic) + 1;
    num_break = Int64(length(findall(sum(qj,dims=1)[:] .!=0)))
    model_size_place = findall(model_size_prior[:,1].==num_param)[1]
    model_size_sim = model_size_prior[model_size_place,2]
    # Marginal likelihood of new model
    lnML_prop = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)


    # Select rows of included variables in break date indicator matrix
    qj_sim_reduced = qj_new[findall([1;xj_in] .!=0),:]

    B_update = gen_B(xj_in,k_in,qj_sim_reduced,B_in,T,k_pot);
    # Compute estimate of error precission
    H_update = H_in;

    ###############################################################
    # Return (1) break date indicator matrix, (2) regression
    # parameter estimaes, (3) Error precision estimate
    ###############################################################
    return qj_new, B_update, H_update, B_in, H_med, lnML_prop, log.(tau_prior[num_break+1]), qa, r

end


function ln_marg_lik_gs(Y::Array{Float64,1}, X::Array{Float64,2}, 
            var_break_ind::Array{Float64,1}, d_vec::Array{Float64,1}, 
            int_prior_var::Float64, m::Float64, v::Float64, 
            mb::Float64, vb::Float64, k_in::Int64, num_break_param::Array{Float64,2})

    flag = 0

    T = length(Y) # Sample Size

    VB = length(findall(var_break_ind .!=0))+1 # Number of variance parameters
    
    K = size(X,2)

    var_break_dates = findall(var_break_ind .!=0) # Dates of structural breaks

    var_break_expand = [0;var_break_dates;T]
    
    e2sum = diff(var_break_expand) # Sample size inside each variance regime

    # Specify intercept and slope parameter priors, conditional on
    # known number of independent variables

    mu = zeros(K) # Prior mean for regression coef.

    # Regression Matrices
    XTX = X'X;
    invXTX = inv(XTX);
    XTY = X'Y


    # Prior variance of regression coef.
    p = deepcopy(K)
    dB = zeros(p)
    dB[1] = int_prior_var
    if p>1
        dB[2:(k_in+1)] .= d_vec[1]
    end
    if p>(k_in+1)
        dB[k_in+2:k_in+1+Int64(num_break_param[1])] .= d_vec[2]
    end
    if p>(k_in+1+Int64(num_break_param[1]))
        dB[k_in+2+Int64(num_break_param[1]):end] .= d_vec[3]
    end
    V = dB.*Matrix{Float64}(I, p, p)
    invV = inv(V)

    # Estimate posterior median of all parameters using Gibbs Sampler

    G0 = 10 # Number of burn-in draws
    G = 100 # Number of draws after convergence
    ND = G0+G # Total number of draws
    
    Btt = X\Y # Initial value of B (OLS estimate)

    # Storage Matrices
    B_sto = zeros(K,G)
    H_sto = zeros(VB,G);
    
    Htt_diag = zeros(T)

    for itr = 1:ND

        Htt = break_model_2.gen_h(Y, X, var_break_dates, Btt, m, v, mb, vb)
    
        htt_1 = Htt[1]

        for jjj=1:(length(var_break_expand)-1)
            Htt_diag[var_break_expand[jjj]+1:var_break_expand[jjj+1]] .= Htt[jjj]
        end

        Htt_diag_rel = Htt_diag./htt_1

        Ystar = Y.*sqrt.(Htt_diag_rel)
        Xstar = X.*sqrt.(Htt_diag_rel)

        Vbar = inv((Xstar'Xstar)+invV)
        mubar = (Vbar*((Xstar'Ystar)+invV*mu))[:]
        Omegabar = Hermitian((1/htt_1)*Vbar)
        Omegabar = convert(Array{Float64,2},Omegabar)

        if isposdef(Omegabar)==false
            flag = 1
            Btt = zeros(length(mubar))
            continue
        end
        Btt = rand(MvNormal(mubar,Omegabar))

        if itr > G0
            B_sto[:,itr-G0] = Btt
            H_sto[:,itr-G0] = Htt
        end

    end

    B_med = zeros(K)

    for i=1:K
        B_med[i] = median(B_sto[i,:][:])
    end

    H_med = zeros(length(var_break_dates)+1)

    for i=1:length(var_break_dates)+1
        H_med[i] = median(H_sto[i,:][:])
    end

    for jjj=1:(length(var_break_expand)-1)
        Htt_diag[var_break_expand[jjj]+1:var_break_expand[jjj+1]] .= H_med[jjj]
    end

    # Compute log likelihood function at posterior median estimates
    e_vec = Y-X*B_med
    e_vec_star = e_vec.*sqrt.(Htt_diag)

    llf = (0.5*sum(log.(Htt_diag)) - 0.5*(e_vec_star'e_vec_star))[1]

    # Compute prior densities at posterior median estimates

    h1 = H_med[1]
    A = Hermitian((1 ./ h1)*V)
    A = convert(Array{Float64,2},A)

    if isposdef(A)==false
        flag=1
        p_B = -1e20
    else
        p_B = logpdf(MvNormal(mu,A),B_med)
    end

    a = v/2
    b = m/a
    p_h1 = logpdf(Gamma(a,b),h1)

    a = vb/2
    b = mb/a
    p_h_break = sum(logpdf.(Gamma(a,b),H_med[2:length(H_med)]))
    p_H = p_h1 + p_h_break
    lprior = p_B + p_H

    # Compute posterior densities at posterior median estimates
    H_diag_rel = Htt_diag./h1
    Ystar = Y.*sqrt.(H_diag_rel)
    Xstar = X.*sqrt.(H_diag_rel)

    Vbar = inv((Xstar'Xstar)+invV)
    mubar = (Vbar*((Xstar'Ystar)+invV*mu))[:]
    Omegabar = Hermitian((1/h1)*Vbar)
    Omegabar = convert(Array{Float64,2},Omegabar)

    if isposdef(Omegabar)==false
        flag = 1
        post_B = 1e20
    else
        post_B = logpdf(MvNormal(mubar,Omegabar),B_med)
    end

    # Estimate p(Hhat|Y) via Monte Carlo Integration
    post_H = 0
    for i=1:G
        Btt = B_sto[:,i]
        p_H = break_model_2.eval_h(Y, X, var_break_dates, H_med, Btt, m, v, mb, vb)
        post_H += p_H
    end

    post_H = post_H/G
    
    lpost = post_B + post_H

    ln_ml_proc = (llf + lprior - lpost)

    # Choose a draw of B and H to store. 
    H_MCMC = zeros(length(var_break_dates)+1)
    for i=1:length(var_break_dates)+1
        H_MCMC[i] = H_sto[i,end]
    end
    for jjj=1:(length(var_break_expand)-1)
        Htt_diag[var_break_expand[jjj]+1:var_break_expand[jjj+1]] .= H_MCMC[jjj]
    end

    B_MCMC = B_sto[:,end]

    return ln_ml_proc, B_MCMC, Htt_diag, H_MCMC, flag

end

function ln_marg_lik(Y::Array{Float64,1},X::Array{Float64,2},d_vec::Array{Float64,1},
int_prior_var::Float64,m::Float64,v::Float64,k_in::Int64, num_break_param::Array{Float64,2})
    
    flag=0
	T = length(Y)
	mu = zeros(size(X,2))
	XTX = X'X
	invXTX = inv(XTX)
	XTY = X'Y

    # Prior variance of regression coef.
    p = size(X,2)
    dB = zeros(p)
    dB[1] = int_prior_var
    if p>1
        dB[2:(k_in+1)] .= d_vec[1]
    end
    if p>(k_in+1)
        dB[k_in+2:k_in+1+Int64(num_break_param[1])] .= d_vec[2]
    end
    if p>(k_in+1+Int64(num_break_param[1]))
        dB[k_in+2+Int64(num_break_param[1]):end] .= d_vec[3]
    end
    V = dB.*Matrix{Float64}(I, p, p)
    Vinv = inv(V)

	# OLS estimates
	Bhat = X\Y
	e_vec = Y-X*Bhat
	SSR = (e_vec'e_vec)[1]
	hhat = T/SSR

	llf = T*0.5*log.(hhat) - 0.5*hhat*SSR

    A = Hermitian((1/hhat)*V)
    A = convert(Array{Float64,2},A)
    p_B = logpdf(MvNormal(mu,A),Bhat)

	a = v/2;
	b = m/a
	p_h1 = logpdf(Gamma(a,b),hhat)

	lprior = p_B + p_h1;

    Vbar = inv(XTX + Vinv)
    mubar = Vbar*(XTY + Vinv*mu)
    Omegabar = Hermitian((1/hhat)*Vbar)
    Omegabar = convert(Array{Float64,2},Omegabar)

    if isposdef(Omegabar)==false
        flag=1
        post_B = 1e20
    else
        post_B = logpdf(MvNormal(mubar,Omegabar),Bhat)
    end

	vbar = T + v
    mbar = vbar/(SSR + (v/m) + (mu-Bhat)'*(inv(V + invXTX))*(mu-Bhat))

	a = vbar/2;
	b = mbar/a;

	post_H = logpdf(Gamma(a,b),hhat)

	lpost = post_B + post_H;

	ln_ml_proc = (llf + lprior - lpost)[1]

	# Compute posterior mean of B
	Bhat = mubar;
	Hhat = mbar.*ones(T,1);

    # Draw value of B for storage
    B_MCMC = rand(MvNormal(mubar,Omegabar))
    H_MCMC = rand(Gamma(a,b)).*ones(T,1);

	return ln_ml_proc, B_MCMC, H_MCMC, flag

end

function permn(V::Array{Int64,1},N::Int64)

nv = length(V)
C = zeros(nv^N,N) # declaration
for ii=1:N
    cc = 1
    for jj=1:(nv^(ii-1))
        for kk=1:nv
            for mm=1:(nv^(N-ii))
                C[cc,ii] = V[kk]
                cc = cc + 1
            end
        end
    end
end
    
return C
end


function gen_B(xj_update::Array{Float64,1},k::Int64,qj_reduced::Array{Float64,2},
B_temp::Array{Float64,1},T::Int64,k_pot::Int64)
	# Computes and stores B_hat for each variable at each point in time

	B_reduced = (B_temp[1:k+1].*ones(k+1,T))' #B_hat for all included variables. Matrix is initiallized at values of B_hat before any breaks

	B0 = B_temp[1:k+1]

	B_temp = B_temp[k+2:end]; # Toss out first k+1 elements of B_temp since they were already used.

	num_break_param = sum(qj_reduced,dims=2); # Computes number of breaks for each parameter

	# Run loop to form the rest of the B_reduced matrix. Similar to the x_proc loop.
	
	ind = 1;
	for jjj = 1:(k+1)
	    if (num_break_param[jjj] != 0)
	        B_indic_temp = findall(qj_reduced[jjj,:] .!=0);
	        B_k_temp = B0[jjj]
	        for qqq = 1:length(B_indic_temp);
	            tau_temp = B_indic_temp[qqq];
	            B_k_temp = B_k_temp+B_temp[ind]
	            if qqq<length(B_indic_temp)
	                B_reduced[tau_temp+1:B_indic_temp[qqq+1],jjj] = B_k_temp.*ones(B_indic_temp[qqq+1]-tau_temp,1);
	            else
	                B_reduced[tau_temp+1:T,jjj] = B_k_temp.*ones(T-tau_temp,1);
	            end
	            ind = ind + 1;
	        end;
	    end; 
	end;

	B_reduced = B_reduced';

	B_update = zeros(k_pot+1,T); # Full B_hat matrix
	if sum(xj_update) > 0
		B_update[[1;findall(xj_update .!=0) .+ 1],:] .= B_reduced; # Put reduced B_hat into right place in the full matrix
	else
		B_update[1,:] = B_reduced
	end

return B_update

end

function gen_h(Y::Array{Float64,1}, X::Array{Float64,2}, 
var_break_dates::Array{Int64,1}, Btt::Array{Float64,1}, 
m::Float64, v::Float64, mb::Float64, vb::Float64)

    T = length(Y)
    e_vec = Y-X*Btt
    e2sum_all = cumsum(e_vec.^2.0)
    e2sum = [[0;e2sum_all[var_break_dates];e2sum_all[T]] [0;var_break_dates;T]];
    e2sum = diff(e2sum,dims=1)

    vbar = e2sum[1,2] + v
    mbar = vbar/(e2sum[1,1]+v/m)

    a = vbar/2
    b = mbar/a
    h1 = rand(Gamma(a,b))

    e2sum_break = e2sum[2:size(e2sum,1),:]

    vbar = e2sum_break[:,2] .+ vb;
    mbar = vbar./(e2sum_break[:,1] .+ vb/mb)

    a = vbar./2
    b = mbar./a

    h_b = zeros(length(a))
    for jjj=1:length(a)
        h_b[jjj] = rand(Gamma(a[jjj],b[jjj]))
    end

    h = [h1;h_b]

    return h
end


function eval_h(Y::Array{Float64,1}, X::Array{Float64,2}, 
var_break_dates::Array{Int64,1}, h_hat_vec::Array{Float64,1}, 
Btt::Array{Float64,1}, m::Float64, v::Float64, mb::Float64, vb::Float64)

    T = length(Y)
    e_vec = Y-X*Btt
    e2sum_all = cumsum(e_vec.^2.0)
    e2sum = [[0;e2sum_all[var_break_dates];e2sum_all[T]] [0;var_break_dates;T]];
    e2sum = diff(e2sum,dims=1)

    vbar = e2sum[1,2] + v
    mbar = vbar/(e2sum[1,1]+v/m)

    a = vbar/2
    b = mbar/a

    p_h = logpdf(Gamma(a,b),h_hat_vec[1])

    e2sum_break = e2sum[2:size(e2sum,1),:]

    vbar = e2sum_break[:,2] .+ vb;
    mbar = vbar./(e2sum_break[:,1] .+ vb/mb)

    a = vbar./2
    b = mbar./a

    lng = 0
    for jjj=1:length(a)
        lng += logpdf.(Gamma(a[jjj],b[jjj]),h_hat_vec[jjj+1])
    end

    p_h = p_h + lng

    return p_h
end

function make_X_break(x_pot::Array{Float64,2},xj::Array{Float64,1},
    qj_sim_reduced::Array{Float64,2},k::Int64,
    num_break_param::Array{Float64,2},T::Int64)
    
    # All included columns of "X"
    x_reduced = [ones(T) x_pot[:,findall(xj .!=0)]]
    # Total number of columns after accounting for break dates
    temp = convert(Int64,k+1+sum(num_break_param))
    # Empty matrix with correct numbers of rows & columns
    x_proc = zeros(T,temp)
    # First "k+1" columns are full "X" columns
    x_proc[:,1:k+1] = x_reduced

    # Counter to keep track of additional columns
    mat_counter = 1
    # Loop through remaining columns. 
    # Only fill the variable in after its corresponding break-date
    for jjj=1:(k+1)
        if (num_break_param[jjj] != 0)
            x_indic_temp = findall(vec(qj_sim_reduced[jjj,:]) .!=0);
            for qqq = 1:length(x_indic_temp)
                tau_temp = x_indic_temp[qqq]
                x_proc[:,k+1+mat_counter] = x_reduced[:,jjj].*([zeros(tau_temp);ones(T-tau_temp)])
                mat_counter = mat_counter+1
            end
        end
    end
    
    return x_proc
end

function msp_compute(k_pot_in, max_m_in)
    # This function computes prior probabilities for models of different  sizes,
    # where model size refers to the number of conditional mean and variance
    # parameters. 

    # Initialize varibales for later use
    num_param = [];
    list_poss_params = [];
    new_mat = [];
    num_poss_params = [];

    for xy = 0:1:k_pot_in
        for m=0:1:max_m_in
            if m==0
                if xy==0
                    num_param = [xy+2 1]
                else
                    num_param = [num_param;[xy+2 1]]
                end
            elseif m==1
                list_poss_params = sum(permn([0;1],xy+2),dims=2)
                list_poss_params = list_poss_params[2:size(list_poss_params,1)]
                num_poss_params = size(list_poss_params,1)

                a = collect(fit(Histogram,list_poss_params,[0;unique(list_poss_params)],closed=:right).edges)[1][2:end]
                b = fit(Histogram,list_poss_params,[0;unique(list_poss_params)],closed=:right).weights

                #(a,b) = hist(list_poss_params,[0;unique(list_poss_params)])

                new_mat = [a b]
                new_mat[:,1] = new_mat[:,1] .+ (xy+2)
                num_param = [num_param;new_mat]
            else
                new_mat_t = kron(ones(size(list_poss_params,1)),new_mat)
                temp_size = convert(Int64,size(new_mat_t,1)/num_poss_params)
                new_mat_t[:,1] = new_mat_t[:,1] + 
                    kron(list_poss_params,ones(temp_size))

                N = length(new_mat_t[:,1])
                A = sortslices([new_mat_t[:,1] collect(1:N)],dims=1)
                IND = convert(Array{Int64,1},A[:,2])
                A = A[:,1]

                new_mat_tt = [A new_mat_t[IND,2]]
                new_mat_u = unique(new_mat_tt[:,1])
                new_mat_u = [new_mat_u zeros(size(new_mat_u,1),1)]
                for jj = 1:1:size(new_mat_u,1)
                    new_mat_u[jj,2] = sum(new_mat_tt[new_mat_tt[:,1].==new_mat_u[jj,1],2])
                end;
                new_mat = deepcopy(new_mat_u);
                num_param = [num_param;new_mat]
            end
        end
    end

    new_mat_t = deepcopy(num_param)
    N = length(new_mat_t[:,1])
    A = sortslices([new_mat_t[:,1] collect(1:N)],dims=1)
    IND = convert(Array{Int64,1},A[:,2])
    A = A[:,1]

    new_mat_tt = [A new_mat_t[IND,2]]
    new_mat_u = unique(new_mat_tt[:,1])
    new_mat_u = [new_mat_u zeros(size(new_mat_u,1),1)]
    for jj = 1:1:size(new_mat_u,1)
        new_mat_u[jj,2] = sum(new_mat_tt[new_mat_tt[:,1].==new_mat_u[jj,1],2])
    end;

    num_param = deepcopy(new_mat_u);

    mod_prior = deepcopy(num_param);
    mod_prior[:,2] = 1 ./ num_param[:,2];
    #mod_prior[:,2] = mod_prior[:,2].*(1/size(mod_prior,1))
    #mod_prior[:,2] = mod_prior[:,2]./sum(mod_prior[:,2])

    return mod_prior
end

end
