#######################################################################
# Script for running BMA for structural break models.
########################################################################

########################################################################
# Load packages and other preliminaries
########################################################################
using Distributions, PyPlot, StatsBase, DataFrames, CSV, Combinatorics, Distributed, SpecialFunctions, 
Random, LinearAlgebra, DelimitedFiles

num_cores = 4
addprocs(num_cores)

@everywhere include("/Users/username/<Insert rest of path to folder holding files>/break_model_cI_prior.jl")

########################################################################
# Read in Data
########################################################################
cd("/Users/username/<Insert rest of path to folder holding data>")
seed_start = 1100

# Data Set
series_name = "UNRATE"
println("Running $(series_name)")

Y = CSV.read("$(series_name).csv")
if (series_name=="no_break") || (series_name=="int_small")
    Y = Y[:,2]
    Y = convert(Array{Float64,1},Y)
else
    Y = convert(Array{Float64,1},Y[:,2])
end
T = length(Y)

start_date = 1959.0
end_date = 2020.0
dates = collect(start_date:0.25:end_date)
dates = dates[1:T]
end_date = dates[end]

# Create potential X Variables
lags = 6 # Maximum number of lags to include in the regression.
x_pot = Y[1:end-lags]
for i=lags-1:-1:1
    j = lags-i+1
    global x_pot = [Y[j:end-i] x_pot]
end

# Redefine Y and T after including lags.
Y = Y[lags+1:end];
T = length(Y)

########################################################################
# Set Model Priors
# Note: I set "max_m=10" to prevent over-fitting.
########################################################################
k_pot = size(x_pot,2)
k_pot = convert(Int64,k_pot)

minp = 8
num_tau = T-(2*minp)+1
num_tau = convert(Int64,num_tau)

max_m = floor(T/minp) .- 1
max_m = convert(Int64,max_m)
max_m = 10

mmm_vec = collect(0:1:max_m)
tau_prior =  (1 ./ (exp.(lgamma.(num_tau .- (mmm_vec .- 1).*(minp) .+ 1) - 
			lgamma.(mmm_vec .+ 1) - lgamma.(num_tau .- (mmm_vec .- 1)*(minp) .- mmm_vec .+1))));

model_size_prior = break_model_2.msp_compute(k_pot,max_m);


########################################################################
# Set Parameter Priors
########################################################################
# Prior on precision of variance break
mb=1.0
vb=0.1

########################################################################
# Prior on B is N(0,V) where V is block diagonal
########################################################################
# V = [1e8 ...
#	   ... d1^2 ...
#	   ... ...  d2^2 ...
#	   ... ...  .... d3^2]
# where d1 is the prior SD on full sample AR terms
# d2 is the prior SD on intercept breaks
# d3 is the prior SD on AR breaks

# Common intercept prior
int_prior_var = 1e8
# d1,d2,d3
c = 1 ./ mean(Chi(1))
d2 = 2.0 ./ (c*4) # For d2 run OLS on prior observarions and set d2 = 4*(1-sum(AR))/(4c)...this implies an SD = 4% annualized
d_vec = [4.0^2;d2.^2;4.0^2]

########################################################################
# Set Other Priors
########################################################################
# Prior on variance precision in all models.
#v = 2.0; m = 5.0 # Implied SD 90% CI = [0.25,1.98]; 99% CI = [0.19,6.26]
v = 1e-10; m = 1.0 # Noninformative prior

########################################################################
# Set Other Starting Values
########################################################################
# Number of draws
G0 = 250000 # Burn-in
G = 1000000/num_cores # Posterior...numerator is total posterior across cores
G = convert(Int64,round(G))
total_draws = G0+G
thin=25; # Note that both G0 and G must be divisible by thin

nbhood = 3;

########################################################################
# Randomly Draw Starting Matrices
########################################################################

# Starting inclusion variables (xj) and breaks (qj)
xj_sim_g = zeros(k_pot,num_cores)
qj_sim_g = zeros(k_pot+2,T,num_cores)

# Loop to randomly draw variable inclusion and breaks
# Each core will start from different variable inclusion,
# break dates, and variable breaks at each date

Random.seed!(seed_start)
for i=1:num_cores
	# Randomly draw number of included variables. These are drawn so
	# higher order AR terms only included if lower AR terms included.

	max_k_init = 2
	vars = rand(DiscreteUniform(0,max_k_init))
	
	# Randomly draw number of breaks
	if max_m > 5
		breaks = rand(DiscreteUniform(0,5))
	else
		breaks = rand(DiscreteUniform(0,max_m))
	end

	# Storage matrices
	break_dates = zeros(breaks)
	var_breaks = zeros(vars+2,breaks)

	if breaks==0
		# If no breaks, set variable inclusion and keep break matrix empty
		xj_sim_g[collect(1:vars),i] = 1
		qj_sim_g[:,:,i] = 0
	else
		# If more than one break, randomly select location of break dates.
		# This does so efficiently by only selecting from eligible dates
		# at each break.

		# Initialize previous break date for use in loop
		prev_break = 1

		# Loop over total number of break dates
		for j=1:breaks
			break_end = T-(breaks+1-j)*minp # Last eligible break date
			break_start = prev_break+minp # First eligible break date

			# Select break date
			break_date = rand(DiscreteUniform(break_start,break_end))
			break_dates[j] = break_date
			prev_break = deepcopy(break_date)

			# Select subset of variables (including intercept & var)
			# that will break at this date

			# Number of variables that will break
			num_var_break = rand(DiscreteUniform(0,vars+2)) 

			# Determine which variables will break
			var_break = sample(collect(1:vars+2),num_var_break,replace=false)
			var_break = sort(var_break)
			var_breaks[var_break,j] .= 1
		end

		# Save variable selection and variable break dates in starting matrices
		break_dates = convert(Array{Int64,1},break_dates)
		xj_sim_g[collect(1:vars),i] .= 1
		qj_sim_reduced = qj_sim_g[[1;collect(1:vars) .+ 1;end],:,i]
		for j=1:breaks
			qj_sim_reduced[findall(var_breaks[:,j] .!=0 ),break_dates[j]] .= 1
		end
		qj_sim_g[[1;collect(1:vars) .+ 1;end],:,i] = qj_sim_reduced
	end
end


########################################################################
# Initialize Models
########################################################################
model_container = break_model_2.break_model[]
for i=1:num_cores
    model = break_model_2.break_model(G0,G,k_pot,T,xj_sim_g[:,i][:],
        qj_sim_g[:,:,i][:,:],x_pot,Y,tau_prior,model_size_prior,d_vec,
        int_prior_var,m,v,mb,vb,minp,max_m,total_draws,nbhood,
        i+seed_start,thin)
    push!(model_container,model)
end

########################################################################
# Run
########################################################################
@time samples_pmap = pmap(break_model_2.gibbs_break,model_container)
#break_model_2.gibbs_break(model_container[1])

figure(figsize=(8,5))
title("Marginal Likelihood by Draw")
for i=1:num_cores
    plot(samples_pmap[i].ln_lik_all,label="Core $(i)")
end
legend(loc="lower right")

########################################################################
# Create Posterior Objects and Redefine G0 & G
########################################################################

# 
println("Break Date Matrix Acceptance Rate by Core")
for i=1:num_cores
    pro = samples_pmap[i].A_accept ./ (G0+G)
    pro = round(pro,digits=4)
    println("Core $(i) = $(pro)")
end
println("")

# 
println("Variable Selection Vector Acceptance Rate by Core")
for i=1:num_cores
    pro = samples_pmap[i].x_accept ./ (G0+G)
    pro = round(pro,digits=4)
    println("Core $(i) = $(pro)")
end
println("")


x_sampled = 0
break_date_sampled=0
for i=1:num_cores
    global x_sampled = x_sampled.+samples_pmap[i].x_sampled
    global break_date_sampled = break_date_sampled.+samples_pmap[i].break_date_sampled
end

k = size(samples_pmap[1].qj_sto,1)
(k_pot,G) = size(samples_pmap[1].x_sto)
(k_pot,G0) = size(samples_pmap[1].x_pre)
G = G.*num_cores
G0 = G0.*num_cores
total_draws = G+G0

cd("/Users/username/<Insert rest of path to storage folder>")
writedlm("Storage/$(series_name)/break_date_sampled.csv",break_date_sampled,',')

#################################################
# Save burn-in draws for convergence diagnostics
#################################################
H_pre = samples_pmap[1].H_pre
for i=2:num_cores
    global H_pre = [H_pre samples_pmap[i].H_pre]
end
writedlm("Storage/$(series_name)/H_pre.csv",H_pre,',')

param_pre = zeros(k-1,T,G0)
dif = convert(Int64,(G0/num_cores)-1)
for i=1:num_cores
    j = convert(Int64,i*(G0/num_cores))
    param_pre[:,:,j-dif:j] = samples_pmap[i].B_pre
end
param_pre[1,:,:] = param_pre[1,:,:]#.+mean_Y
writedlm("Storage/$(series_name)/param_pre.csv",param_pre[:],',')

qj_pre = zeros(k,T,G0)
dif = convert(Int64,(G0/num_cores)-1)
for i=1:num_cores
    j = convert(Int64,i*(G0/num_cores))
    qj_pre[:,:,j-dif:j] = samples_pmap[i].qj_pre
end
writedlm("Storage/$(series_name)/qj_pre.csv",qj_pre[:],',')

xj_pre = zeros(k-2,G0)
dif = convert(Int64,(G0/num_cores)-1)
for i=1:num_cores
	j = convert(Int64,i*(G0/num_cores))
	xj_pre[:,j-dif:j] = samples_pmap[i].x_pre
end
writedlm("Storage/$(series_name)/xj_pre.csv",xj_pre,',')


########################################################################
# Save posterior draws
########################################################################

H_mat = samples_pmap[1].H_sto
for i=2:num_cores
    global H_mat = [H_mat samples_pmap[i].H_sto]
end
writedlm("Storage/$(series_name)/H.csv",H_mat,',')

param_mat = zeros(k-1,T,G)
dif = convert(Int64,(G/num_cores)-1)
for i=1:num_cores
    j = convert(Int64,i*(G/num_cores))
    param_mat[:,:,j-dif:j] = samples_pmap[i].B_sto
end
param_mat[1,:,:] = param_mat[1,:,:]#.+mean_Y
writedlm("Storage/$(series_name)/param_mat.csv",param_mat[:],',')

Y_hat_mat = zeros(T,G)
X = [ones(T) x_pot]
for i=1:G
    Y_hat_mat[:,i]=diag(X*param_mat[:,:,i][:,:])
end

qj_total = zeros(k,T,G)
dif = convert(Int64,(G/num_cores)-1)
for i=1:num_cores
    j = convert(Int64,i*(G/num_cores))
    qj_total[:,:,j-dif:j] = samples_pmap[i].qj_sto
end
writedlm("Storage/$(series_name)/qj.csv",qj_total[:],',')
#qj_total = readcsv("Storage/$(series_name)/qj.csv")
#qj_total = reshape(qj_total,(k,T,G))

xj_total = zeros(k-2,G)
dif = convert(Int64,(G/num_cores)-1)
for i=1:num_cores
	j = convert(Int64,i*(G/num_cores))
	xj_total[:,j-dif:j] = samples_pmap[i].x_sto
end
writedlm("Storage/$(series_name)/xj.csv",xj_total,',')

ML_sto = zeros(G)
dif = convert(Int64,(G/num_cores)-1)
for i=1:num_cores
    j = convert(Int64,i*(G/num_cores))
    ML_sto[j-dif:j] = samples_pmap[i].ln_lik_sto
end
writedlm("Storage/$(series_name)/ML_sto.csv",ML_sto,',')

########################################################################
# Compute and Print Posterior Stats
########################################################################
dates = collect(start_date:0.25:end_date)
dates = dates[lags+1:end]

names = ["Intercept"]
for i=2:k-1
    global names = [names;"Variable $(i-1)"]
end
names = [names;"Conditional Variance"]

seeds = zeros(num_cores)
for i=1:num_cores
    seeds[i] = samples_pmap[i].seed
end
writedlm("Storage/$(series_name)/seeds.csv",seeds,',')


########################################################################
# Minimum Break Date Constraint
########################################################################
println("")
min_count = 0
for j=1:G
    break_place = findall(sum(qj_total[:,:,j],dims=1)[:] .!=0)
    if sum(diff(break_place).==(minp+1))>0
    	global min_count+=1
    end
end
min_count = round(100 .* (min_count ./ (G)),digits = 2)
println("Percentage of Posterior Draws Where Minimum Break Date Constraint Binds")
println("$(min_count)%")
println("")

########################################################################
# Mean and Median DRAWN break dates
########################################################################
median_breaks = zeros(k,1)
mean_breaks = zeros(k,1)

qj_vec_sto = sum(samples_pmap[1].qj_sto,dims=2)
qj_vec_sto = reshape(qj_vec_sto,(size(qj_vec_sto)[1],size(qj_vec_sto)[end]))

for i=2:num_cores
	qj_vec_temp = sum(samples_pmap[i].qj_sto,dims=2)
	qj_vec_temp = reshape(qj_vec_temp,(size(qj_vec_temp)[1],size(qj_vec_temp)[end]))

    global qj_vec_sto = [qj_vec_sto qj_vec_temp]
end

for i=1:k
    median_breaks[i] = quantile(qj_vec_sto[i,:][:],0.5)
end
mean_breaks = mean(qj_vec_sto,dims=2)


########################################################################
# Break Date Correlations Across Cores
########################################################################

break_by_core = samples_pmap[1].break_date_sampled[:]
for i=2:num_cores
	global break_by_core = [break_by_core samples_pmap[i].break_date_sampled[:]]
end

println("")
display("Correlation of Break Dates across Cores")
display(cor(break_by_core))

A = [dates break_by_core]
writedlm("Storage/$(series_name)/break_by_core.csv",A,',')

#######################################################################
# Compute Stats using Analytical Model Probabilities
#######################################################################
# Create matrix that contains location of breaks
qj = deepcopy(qj_total)
xj = deepcopy(xj_total)

# Find number of parameter breaks in each model
l_sto = zeros(G)
for i=1:G
    l_sto[i] = length(findall(qj[:,:,i] .!= 0))
end
l_max = convert(Int64,maximum(l_sto)) # Maximum number of breaks in any model
q_place_sto = zeros(l_max,G) # Storage matrix for break dates

for i=1:G
    l = convert(Int64,l_sto[i])
    place = findall(vec(qj[:,:,i]) .!= 0)
    if l>0
        q_place_sto[1:l,i] .= place
    end
end
q_place_sto = convert(Array{Int64,2},q_place_sto)


#######################################################################
# Find all unique models
#######################################################################
model =[1]
q_unique = collect(q_place_sto[:,1])
x_unique = collect(xj[:,1])
times = [sum(sum(q_place_sto[:,1].==q_place_sto,dims=1)[:] .==l_max)]
times = [1]

ln_lik_all = samples_pmap[1].ln_lik_sto
for i=2:num_cores
    global ln_lik_all = [ln_lik_all;samples_pmap[i].ln_lik_sto]
end

ln_lik_from_run = ln_lik_all[1]

for i=2:G
    q = q_place_sto[:,i]
    x = xj[:,i]
    lnML = ln_lik_all[i]
    if ((sum(sum(q.==q_unique,dims=1)[:] .==l_max)) == 0)
            global model =[model;model[end]+1]
            global q_unique = [q_unique q]
            global x_unique = [x_unique x]
            global times = [times;1]
            global ln_lik_from_run = [ln_lik_from_run;lnML]
    elseif ((sum(sum(q.==q_unique,dims=1)[:] .==l_max)) > 0)
        unique_vec = findall(sum(q.==q_unique,dims=1)[:] .== l_max)
        if length(unique_vec)==1
            j = findall(sum(q.==q_unique,dims=1)[:] .== l_max)[1]
            if sum(x.==x_unique[:,j],dims=1)[:][1].==k_pot
                times[j] = times[j]+1
            else
                global model =[model;model[end]+1]
                global q_unique = [q_unique q]
                global x_unique = [x_unique x]
                global times = [times;1]
                global ln_lik_from_run = [ln_lik_from_run;lnML]
            end
        else
            itr = 0
            for k=1:length(unique_vec)
                j = findall(sum(q.==q_unique,dims=1)[:] .== l_max)[k]
                if sum(x.==x_unique[:,j],dims=1)[:][1].==k_pot
                    times[j] = times[j]+1
                    itr+=1
                end
            end
            if itr==0
                global model =[model;model[end]+1]
                global q_unique = [q_unique q]
                global x_unique = [x_unique x]
                global times = [times;1]
                global ln_lik_from_run = [ln_lik_from_run;lnML]
            end
        end 
    end
end
probs = times./G

# Use full list and compute analytical model probabilities
G1 = length(probs)

println("")
println("Number of unique models visited by the sampler = $(G1)")
println("")

#######################################################################
# Create Analytical Probabilities for All Unique Models
#######################################################################

k_pot = size(x_pot,2)
ln_lik_sto = zeros(G1)

print("Computing analytical probabilities.")
gg1 = Int64(round(G1./20,digits=0))
for i=1:G1
    if mod(i,gg1)==0
        print(".")
    end

    q = q_unique[:,i]
    qq = zeros(k_pot+2,T)
    qq[q[findall(q .!=0)]] .= 1.0
    qj_in = deepcopy(qq)
    
    xj_in = x_unique[:,i]
    k_in = convert(Int64,sum(xj_in))

    qj_reduced = qj_in[findall([1;xj_in;1] .!=0),:]
    (kk,TT) = size(qj_reduced)

    # Find marginal likelihood of current model
    num_break_param = sum(qj_reduced[1:k_in+1,:],dims=2)
    x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_reduced,k_in,num_break_param,TT)
    var_break_indic = qj_reduced[k_in+2,:][:]
    if sum(var_break_indic)==0
        (ln_fy_proc,B_med,Htt_diag,flag) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        else
        (ln_fy_proc,B_med,Htt_diag,H_med) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end
    num_param = size(x_proc,2) + sum(var_break_indic) + 1;
    num_break = Int64(length(findall(sum(qj_reduced,dims=1)[:] .!=0)))
    model_size_place = findall(model_size_prior[:,1].==num_param)[1]
    model_size_sim = model_size_prior[model_size_place,2]
    lnML_current = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)
    ln_lik_sto[i] = lnML_current
end
println("")
max_ln = maximum(ln_lik_sto)
lik = exp.(ln_lik_sto.-max_ln)
analytic_probs = (lik./sum(lik))

#######################################################################
# Use analytic probabilities to sample from model space
#######################################################################
x = collect(1:1:G1)
M = 10000
mm1 = Int64(round(M./20,digits=0))
B_sampled = zeros(k_pot+1,T,M)
H_sampled = zeros(T,M)
AR_sampled = zeros(T,M)
Y_hat_sampled = zeros(T,M)

print("Sampling from model space.")
for itr=1:M
    if mod(itr,mm1)==0
        print(".")
    end

    i = sample(x,Weights(analytic_probs))
    q = q_unique[:,i]
    qq = zeros(k_pot+2,T)
    qq[q[findall(q .!=0)]] .= 1.0
    qj_in = deepcopy(qq)
    
    xj_in = x_unique[:,i]
    k_in = convert(Int64,sum(xj_in))

    qj_reduced = qj_in[findall([1;xj_in;1] .!=0),:]
    (kk,TT) = size(qj_reduced)

    # Find marginal likelihood of current model
    num_break_param = sum(qj_reduced[1:k_in+1,:],dims=2)
    x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_reduced,k_in,num_break_param,TT)
    var_break_indic = qj_reduced[k_in+2,:][:]
    if sum(var_break_indic)==0
        (ln_fy_proc,B_med,Htt_diag,flag) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
        else
        (ln_fy_proc,B_med,Htt_diag,H_med) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
    end

    B_vec = break_model_2.gen_B(xj_in,k_in,qj_reduced,B_med,T,k_pot)
    H_vec = deepcopy(Htt_diag)
    AR_sum = sum(B_vec[2:end,:],dims=1)[:]
    Yhat = x_proc*B_med

    B_sampled[:,:,itr] = B_vec
    H_sampled[:,itr] = H_vec
    AR_sampled[:,itr] = AR_sum
    Y_hat_sampled[:,itr] = Yhat
end
println("")

#######################################################################
# Find all stationary draws
#######################################################################
stationary = zeros(M)

print("Finding all stationary draws.")
for i=1:M
    if mod(i,mm1)==0
        print(".")
    end
    s_temp = ones(T)
    for j=1:T
        F = zeros(k_pot,k_pot)
        F[1,:] = B_sampled[2:end,j,i][:]
        F[2:end,1:end-1] = Matrix{Float64}(I, k_pot-1, k_pot-1)
        max_eig = maximum(abs.(eigvals(F)[1]))
        if max_eig>0.995
            s_temp[j] = 0
        end
    end

    if sum(s_temp)==T
        stationary[i] = 1
    end
end
println("")

percent_stationary = sum(stationary)./M
println("Posterior Probability of Stationarity = $(round(100*percent_stationary,digits=2))%")
println("")

B_stationary = B_sampled[:,:,findall(stationary.==1)]
AR_stationary = AR_sampled[:,findall(stationary.==1)]
unc_mean = B_stationary[1,:,:] ./ (1 .-AR_stationary)
unc_mean_annual = 100 .* ((1 .+ (B_stationary[1,:,:] ./ (1 .-AR_stationary)) ./ 100) .^4 .-1)

H_stationary = H_sampled[:,findall(stationary.==1)]
Y_hat_stationary = Y_hat_sampled[:,findall(stationary.==1)]


##########################################################################################
# Compute correlation between MC3 and analytical probabilities 
##########################################################################################
model_correlation = cor(probs,analytic_probs)
mc = round(model_correlation,digits=3)
println("")
println("Model Correlation = $mc")

A = sortslices([round.(100 .* analytic_probs,digits=2) round.(100 .* probs,digits=2) model],dims=1,rev=true)[1:15,:]
A = ["Analytic" "MC3" "Model Number"; A]


q_analytical = 0
mmm_break = zeros(max_m+1)
for i=1:G1
    q = q_unique[:,i]
    qq = zeros(k_pot+2,T)
    qq[q[findall(q .!=0)]] .= 1.0
    qj_in = deepcopy(qq)
    global q_analytical += length(findall(sum(qj_in,dims=1)[:] .>0)).*analytic_probs[i]

    num_breaks = Int64(length(findall(sum(qj_in,dims=1)[:] .>0)))
    mmm_break[num_breaks+1] += analytic_probs[i]
end

q_drawn = 0
for i=1:G
    qj_in = qj[:,:,i]
    global q_drawn += length(findall(sum(qj_in,dims=1)[:] .>0))
end
q_drawn = q_drawn./G

# Average Number of break dates
total_break_dates = ["Analytical" "Drawn"; q_analytical q_drawn]
println("")
display("Avg. Number of Break Dates")
display(total_break_dates)
println("")

# Posterior Probability of Alternative Numbers of Breaks
println("Posterior Probability of Alternative Numbers of Breaks, Analytical")
    for break_itr = 0:max_m
        a = mmm_break[break_itr+1]
        println(" $break_itr breaks:  $a"); 
    end;
println("")

q_analytical = zeros(lags+2,T)
x_analytical = zeros(lags)
for i=1:G1
    q = q_unique[:,i]
    qq = zeros(k_pot+2,T)
    qq[q[findall(q .!=0)]] .= 1.0
    qj_in = deepcopy(qq)
    global q_analytical += qj_in.*analytic_probs[i]

    xj_in = x_unique[:,i]
    global x_analytical += xj_in.*analytic_probs[i]
end

analytical_break_by_var = sum(q_analytical,dims=2)[:]
drawn_break_by_var = sum(mean(qj,dims=3),dims=2)[:]
varNames = ["Intercept"]
for i=1:lags
    global varNames = [varNames;"AR($(i))"]
end
varNames = [varNames;"Variance"]

drawn_inclusion = mean(xj,dims=2)
inclusion_compare = ["Analytical" "Drawn"; x_analytical drawn_inclusion]
inclusion_compare = [["Variable";varNames[2:end-1]] inclusion_compare]
println("")
display("Variable Inclusion Probabilities")
display(inclusion_compare)

println("")
break_by_var_compare = [varNames analytical_break_by_var drawn_break_by_var]
break_by_var_compare = ["Variable" "Analytical" "Drawn";break_by_var_compare]
display("Mean Number of Breaks by Variable")
display(break_by_var_compare)

